;;; -*- Lisp -*-
;;;
;;; connection from cmn to the (minimal) window handler in nps.lisp
;;;

#+(or KCL Excl) (in-package :cmn :use '(:lisp :pcl :loop))
#+Lucid (in-package :cmn :use '(:lisp))

;;; Ties into CMN are via before/after/around methods.  Nothing in
;;; cmn0..4 depends on anything here.


;;; under title bar, save  zoom in   delete  vrule[]  redraw   page  print
;;;                  undo  zoom out  edit    hrule[]  justify  line  score

(defvar current-active-window nil)

(defstruct menu 
  save-rect undo-rect zoom-in-rect zoom-out-rect delete-rect edit-rect
  vrule-rect hrule-rect redraw-rect justify-rect page-rect line-rect
  print-rect score-rect overall-rect all-rects)

(defun create-menu (window)
  (let* ((wx0 (window-x0 window))
	 (wx1 (window-x1 window))
	 (wy1 (window-y1 window))
	 (scroll-base (if (window-vertical-scroll-rect window) 18 0))
	 (m0y0 (- wy1 22 18 1))
	 (m0y1 (- wy1 22 1))
	 (m1y0 (- wy1 22 18 18 1 1))
	 (m1y1 (- wy1 22 18 1 1))
	 (mwid (floor (- wx1 wx0 scroll-base) 7))
	 (menu (make-menu 
		:save-rect (make-rect :data "Save"
				      :x0 (+ wx0 scroll-base) :y0 m0y0 :x1 (+ wx0 scroll-base mwid -8) :y1 m0y1
				      :code #'(lambda () 
						(when (window-unsaved-changes window)
						  (cmn-store *cmn-score* "tmp.tmp")
;						  (load "tmp.tmp")
						  (setf (window-unsaved-changes window) nil)
						  (close-button-image (- (window-x1 window) 20) (- (window-y1 window) 20))
						  )))
;						  (clear-window window)
;						  (dps-display-file (output-file *cmn-score*)))))

		:zoom-in-rect (make-rect :data "Zoom In" 
					 :x0 (+ 1 wx0 scroll-base mwid -8) :y0 m0y0 :x1 (+ wx0 scroll-base (* 2 mwid)) :y1 m0y1)

		:delete-rect (make-rect :data "Delete" 
					:x0 (+ 1 wx0 scroll-base (* 2 mwid)) :y0 m0y0 :x1 (+ wx0 scroll-base (* 3 mwid)) :y1 m0y1)

		:vrule-rect (make-rect :data "Vrule" 
				       :x0 (+ 1 wx0 scroll-base (* 3 mwid)) :y0 m0y0 :x1 (+ wx0 scroll-base -6 (* 4 mwid)) :y1 m0y1)

		:redraw-rect (make-rect :data "Redraw" 
					:x0 (+ 1 wx0 scroll-base -6 (* 4 mwid)) :y0 m0y0 :x1 (+ wx0 scroll-base (* 5 mwid)) :y1 m0y1
					:code  #'(lambda () 
						   (redrawify *cmn-score* 
							      current-active-window 
							      (if current-page 
								  (dpy-page-page current-page)
								1))))

		:page-rect (make-rect :data "Page" 
				      :x0 (+ 1 wx0 scroll-base (* 5 mwid)) :y0 m0y0 :x1 (+ wx0 scroll-base (* 6 mwid)) :y1 m0y1)

		:print-rect (make-rect :data "Print" 
				       :x0 (+ 1 wx0 scroll-base (* 6 mwid)) :y0 m0y0 :x1 (- wx1 1) :y1 m0y1
				       :code #'(lambda () ;apparently it's safe to lpr on a machine without a printer
						 (excl:shell "lpr ~A~%" (if current-page
									    (dpy-page-filename current-page)
									  "aaa.eps"))))

		:undo-rect (make-rect :data "Undo" 
				      :x0 (+ wx0 scroll-base) :y0 m1y0 :x1 (+ wx0 scroll-base mwid -8) :y1 m1y1)

		:zoom-out-rect (make-rect :data "Zoom Out" 
					  :x0 (+ 1 wx0 scroll-base mwid -8) :y0 m1y0 :x1 (+ wx0 scroll-base (* 2 mwid)) :y1 m1y1)

		:edit-rect (make-rect :data "Edit" 
				      :x0 (+ 1 wx0 scroll-base (* 2 mwid)) :y0 m1y0 :x1 (+ wx0 scroll-base (* 3 mwid)) :y1 m1y1)

		:hrule-rect (make-rect :data "Hrule" 
				       :x0 (+ 1 wx0 scroll-base (* 3 mwid)) :y0 m1y0 :x1 (+ wx0 scroll-base -6 (* 4 mwid)) :y1 m1y1)

		:justify-rect (make-rect :data "Justify" 
					 :x0 (+ 1 wx0 scroll-base -6 (* 4 mwid)) :y0 m1y0 :x1 (+ wx0 scroll-base (* 5 mwid)) :y1 m1y1)

		:line-rect (make-rect :data "Line" 
				      :x0 (+ 1 wx0 scroll-base (* 5 mwid)) :y0 m1y0 :x1 (+ wx0 scroll-base (* 6 mwid)) :y1 m1y1)

		:score-rect (make-rect :data "Score" 
				       :x0 (+ 1 wx0 scroll-base (* 6 mwid)) :y0 m1y0 :x1 (- wx1 1) :y1 m1y1
				       :code #'(lambda () 
						 (setf current-active-rectangle nil)
						 (setf current-active-object *cmn-score*)))
		)))

    (setf (menu-overall-rect menu) (make-rect :x0 (+ wx0 scroll-base) :y0 m1y0 :x1 (- wx1 1) :y1 m0y1))
    (setf (window-menu-rect window) (menu-overall-rect menu))
    (setf (menu-all-rects menu)
      (list (menu-save-rect menu) (menu-undo-rect menu) (menu-zoom-in-rect menu) (menu-zoom-out-rect menu) 
	    (menu-delete-rect menu) (menu-edit-rect menu) (menu-vrule-rect menu) (menu-hrule-rect menu) 
	    (menu-redraw-rect menu) (menu-justify-rect menu) (menu-page-rect menu) (menu-line-rect menu) 
	    (menu-print-rect menu) (menu-score-rect menu)))
    (setf (window-menu-rects window) (menu-all-rects menu))
    ))

  ;; zoom-in/out should be just a matter of resetting the scale (via scale-window), and keep
  ;;   the current point (mx/my) in the same relative place in the window
  ;; edit can use identify
  ;; delete can delete the current-active-object from *cmn-score*
  ;; undo needs a history/edit list
  ;; vrule/hrule need to place/erase the rulers and prepare (activate) the various tab stops etc
  ;; justify calls cmn from the top
  ;; page inserts a page mark at mx-my (similarly for line)

  ;; double-click on score might normalize everything back to starting point
  ;; shift-click on some objects might treat mouse drag as stretch/compress (staff, slur, tie, beam)

  ;; in fact, we might put up an edit menu in place of the basic window menu
  ;; (grow/shrink  darker/lighter  stretch/compress  rotate/mirror  ...)
  ;; or worse, we could have a special menu for each class

(defvar all-pages nil)
(defvar current-page nil)
(defvar current-rectangles nil)
(defvar current-rectangle-id 0)
(defvar max-depth 0)
(defun rectangle-id () (incf current-rectangle-id))

(defun update-current-rectangles (server sx sy) (update-rectangles server sx sy current-rectangles))

(defclass rectify () ((rectangle :initarg :rectangle :initform nil :accessor rectangle)))
(defmethod house ((rect rectify) score) (declare (ignore score)) nil)
(defmethod copy ((rect rectify) &optional wo) (declare (ignore wo)) nil)
(defmethod x0 ((rect rectify)) (rect-x0 (rectangle rect)))
(defmethod y0 ((rect rectify)) (rect-y0 (rectangle rect)))
(defmethod x1 ((rect rectify)) (rect-x1 (rectangle rect)))
(defmethod y1 ((rect rectify)) (rect-y1 (rectangle rect)))
(defmethod identify ((rect rectify)) "") ;(nps-rectangle)")
(defmethod display ((rect rectify) container score &rest rest) (declare (ignore container score rest)) nil)
(defun rectify-p (obj) (typep obj 'rectify))
(defmethod descry ((rect rectify) &optional stream controller) 
  (declare (ignore controller))
  (format stream " (nps-rectangle ~1,3F ~1,3F ~1,3F ~1,3F)" (x0 rect) (y0 rect) (x1 rect) (y1 rect)))

(defun nps-rectangle (&rest args) (declare (ignore args)) nil)

(defvar x-boundary 2)
(defvar y-boundary 2)

(defun add-boundaries (rt)
  (decf (rect-x0 rt) x-boundary)
  (decf (rect-y0 rt) y-boundary)
  (incf (rect-x1 rt) x-boundary)
  (incf (rect-y1 rt) y-boundary)
  rt)

(defun add-rectangle (rt)		;sometimes display just defers -- slurs for example
  (if (and rt (rect-x0 rt)) (push (add-boundaries rt) current-page))
  rt)

(defvar auto-display t)			;if t -> automatically display page 1 of the result of cmn

(defstruct dpy-page filename page box data)
(defvar dpy-pages nil)

(defmethod finalize :before ((server server) &key)
  (when auto-display
    (push (make-dpy-page :filename (output-file server)
			 :box (list (x0 server) (y0 server) (x1 server) (y1 server))
			 :page *cmn-page*
			 :data (sort-page current-page))
	  dpy-pages)
    (setf current-page nil)))

(defmethod cmn-initialize :before (anything) 
  (declare (ignore anything))
  (when (and auto-display (not no-initialization))
    (if current-active-window (error "recursive cmn call with active display does not work yet -- will crash the machine"))
    (setf current-rectangle-id 0)
    (setf max-depth 0)
    (setf dpy-pages nil)))

(defmethod cmn-finalize :after (anything)
  (declare (ignore anything))
  (when (and auto-display (not no-initialization))
    (setf dpy-pages (nreverse dpy-pages))
    (cmndpy (dpy-page-filename (first dpy-pages)))))


(defvar show-tracking-rectangles nil)

(defmethod display :before ((vis visible-mixin) container score &rest rest)
  (when (and auto-display (not (intersection '(:just-fooling :animation) rest)))
    (push (make-rect :obj vis :owner container :matrix (copy-list (scaling-matrix score))) current-rectangles)))

(defmethod display :after ((vis visible-mixin) container score &rest rest)
  (declare (ignore container))
  (when auto-display 
    (if (not (intersection '(:just-fooling :animation) rest))
	;; add-rectangle gives us a pointer from the rectangle to its object
	;; and add-to-marks of a rectify object gives a pointer from the object to its rectangle
	(let ((rt (add-rectangle (pop current-rectangles))))
	  (when (and rt (rect-x0 rt))
	    (setf (rect-id rt) (rectangle-id))
	    (setf (rect-depth rt) (length current-rectangles))
	    (setf max-depth (max max-depth (rect-depth rt)))
	    (add-to-marks vis (list (make-instance 'rectify :rectangle rt)))))
      (when (and show-tracking-rectangles (member :redraw rest))
	(let ((rct (back-rectangle vis)))
	  (if rct
	      (format (output score) "~1,3F setgray ~D ~D ~D ~D rectstroke 0 setgray~%"
		      (if (zerop (rect-depth rct)) .75
			(if (= 1 (rect-depth rct)) .5
			  .25))
		      (rect-x0 rct) (rect-y0 rct)
		      (- (rect-x1 rct) (rect-x0 rct))
		      (- (rect-y1 rct) (rect-y0 rct)))))))))


(defmethod lineto :before ((server server) x y)
  (declare (ignore x y))
  (when current-rectangles (update-current-rectangles server (x server) (y server))))
(defmethod lineto :after ((server server) x y)
  (declare (ignore x y))
  (when current-rectangles (update-current-rectangles server (x server) (y server))))

(defmethod rlineto :before ((server server) dx dy)
  (declare (ignore dx dy))
  (when current-rectangles (update-current-rectangles server (x server) (y server))))
(defmethod rlineto :after ((server server) dx dy)
  (declare (ignore dx dy))
  (when current-rectangles (update-current-rectangles server (x server) (y server))))

(defmethod curveto :before ((server server) x0 y0 x1 y1 x2 y2)
  (declare (ignore x0 y0 x1 y1 x2 y2))
  (when current-rectangles (update-current-rectangles server (x server) (y server))))
(defmethod curveto :after ((server server) x0 y0 x1 y1 x2 y2)
  (declare (ignore x0 y0 x1 y1 x2 y2))
  (when current-rectangles (update-current-rectangles server (x server) (y server))))

(defmethod nps-rx ((glf glyph-mixin) dx) (declare (ignore dx)) (rx glf))
(defmethod nps-rx ((glo glyph-list) dx) (loop for obj in (data glo) sum (+ (rx obj) (or dx 0))))
(defmethod nps-rx ((text text) dx) (* (length (letters text)) (+ .4 (or dx 0))))

(defmethod nps-ry ((glf glyph-mixin)) (y1 glf))
(defmethod nps-ry ((glo glyph-list)) (loop for obj in (data glo) maximize (y1 obj)))
(defmethod nps-ry ((text text)) 1.0)

(defmethod simple-show :before ((server server) glf)
  (when current-rectangles 
    (update-current-rectangles server (x server) (y server))
    (let ((nps-size (size server)))
      (update-current-rectangles server (x server) (+ (y server) (* (y0 glf) nps-size)))
      (update-current-rectangles server 
			 (+ (x server) (* (rx glf) nps-size)) 
			 (+ (y server) (* (y1 glf) nps-size))))))

(defmethod just-line :before ((server server) x0 y0 x1)
  (declare (ignore x0 y0 x1))
  (when current-rectangles 
    (update-current-rectangles server (* (size server) x0) (* (size server) y0))
    (update-current-rectangles server (* (size server) x1) (* (size server) y0))))

(defmethod just-stem :before ((server server) x0 y0 x1)
  (declare (ignore x0 y0 x1))
  (when current-rectangles 
    (update-current-rectangles server (* (size server) x0) (* (size server) y0))
    (update-current-rectangles server (* (size server) x1) (* (size server) y0))))

(defmethod slanted-fill-in :before ((server server) x0 y0 x1 y1 thickness)
  (declare (ignore x0 y0 x1 y1 thickness))
  (when current-rectangles 
    (update-current-rectangles server (* (size server) x0) (* (size server) y0))
    (update-current-rectangles server (* (size server) x1) (* (size server) y1))))

(defmethod optimize-stem :before ((server server) x0 y0 y1 width)
  (declare (ignore x0 y0 y1 width))
  (when current-rectangles 
    (update-current-rectangles server (* (size server) x0) (* (size server) y0))
    (update-current-rectangles server (* (size server) x0) (* (size server) y1))))

(defmethod show :before ((server server) anything &key (dx 0) (dy 0) font size scaler (save :both) &allow-other-keys)
  (declare (ignore save font dy))
  (when current-rectangles 
    ;; this is a bit messy because x/y server are usually not updated by a show (i.e. gsave intervenes)
    ;; -- we have glyph-mixin, glyph-list, and text here (under "anything")
    (update-current-rectangles server (x server) (y server))
    ;; first pick up the untransformed box
    (let* ((dx0 0)
	   (dy0 (if (typep anything 'glyph-mixin) (y0 anything)
		  (if (typep anything 'glyph-list) (loop for obj in (data anything) minimize (y0 obj))
		    0)))
	   (dx1 (nps-rx anything dx))
	   (dy1 (nps-ry anything))
	   (nps-size (if size size 
		       (if scaler (* scaler (size server)) 
			 (if (text-p anything)
			     (or (font-size anything)
				 (* (font-scaler anything) (size server)))
			   (size server)))))
	   (nps-matrix (%scale-matrix (matrix anything) nps-size nps-size))
	   (new-box (transform-box nps-matrix (list 0 0) (list dx0 dy0 dx1 dy1))))
      ;;(print (format nil "obj: ~A, mat: ~A, new box: ~A" anything nps-matrix new-box))
      (update-current-rectangles server (+ (x server) (first new-box)) (+ (y server) (second new-box)))
      (update-current-rectangles server (+ (x server) (third new-box)) (+ (y server) (fourth new-box))))))

(defun back-rectangle (obj)
  (let ((rct (find-if #'rectify-p (marks obj))))
    (if rct
	(rectangle rct))))

(defgeneric incorporate-changes (object owner dx dy) )
;;; for every object in a score that can be independently manipulated by the mouse, we need
;;; a way to take into account the mouse changes so that future displays/edits will "do the right thing"
;;; For a note, for example, this means fixing up pitch info if dy changes.
;;; the method should return nil unless it thinks a massive redisplay is called for.

(defmethod incorporate-changes (anything owner dx dy)
  (declare (ignore owner))
  (incf (dx anything) dx)
  (incf (dy anything) dy)
  nil)

(defmethod incorporate-changes ((stf staff-mixin) owner dx dy)
  (declare (ignore owner))
  (let* ((old-stf-y0 (staff-y0 stf))
	 (stf-y0 (+ dy old-stf-y0)))
    (incf (dx stf) dx)
;    (incf (dy stf) dy)			;need to keep info in subsequent save/redisplay
    ;; this needs to be saved in the vertical ruler (line data record) for this line
;    (when (not (zerop dy))
;      (setf (staff-y0 stf) stf-y0)
      (loop for obj in (staff-data (back-staff stf)) do
	(when (and (score-object-p obj) (= (staff-y0 obj) old-stf-y0))
	  (if (not (staff-p obj))
	      (edit-rectangle (back-rectangle obj) (* (size *cmn-score*) dx) (* (size *cmn-score*) dy)))
;; what about dx?
	  (setf (staff-y0 obj) stf-y0)
	  (when (audible-p obj)
	    (if (chord-p obj)
		(loop for note in (chord-data obj) do
		  (setf (staff-y0 note) stf-y0))
	      (if (beams obj)
		  (setf (staff-y0 (beams obj)) stf-y0)))))))
;)
  (not (zerop dy)))


(defmethod comment :around ((server server) x) 
  (declare (ignore x)) 
  nil)					; DPS is confused by comments

(defun object-image (obj owner)
  (let ((str (make-string-output-stream)))
    (setf (output *cmn-score*) str)
    (display obj owner *cmn-score* :animation)
    (get-output-stream-string str)))

(defvar current-active-rectangle nil)
(defvar current-active-object nil)

(defun send-message (val)
  ;; if there's an active object and a message is typed, it is applied to the currently active object
  (if (self-acting-p val)
      (if current-active-rectangle
	  (let* ((nval (funcall (action val) (rect-obj current-active-rectangle) (arguments val)))
		 (rct current-active-rectangle)
		 (obj (rect-obj rct))
		 (old-image (rect-code rct))
		 (animated-matrixless-image (and obj (object-image obj (rect-owner rct))))
		 (animated-image (if animated-matrixless-image
				     (if (rect-matrix rct)
					 (format nil "gsave [~{~1,3F ~}] concat~%~A~%grestore~%"
						 (rect-matrix rct) animated-matrixless-image)
				       animated-matrixless-image))))
	    (setf (rect-code rct) animated-image)
	    (dps-send (format nil "1 setgray ~A .667 setgray ~A 0 setgray~%" old-image animated-image))
	    nval)
	(if current-active-object
	    (funcall (action val) current-active-object (arguments val))
	  (error "no active object for ~A" val)))
    val))

;;; since global scaling stays the same over many objects (an entire staff normally),
;;; redrawify can save itself lots of matrix operations by checking matrix equality.

(defun redrawify (score win page)
  ;; use :animation and set current-rectangles to nil for duration
  (let ((old-rects current-rectangles)
	(str (make-string-output-stream)))
    (setf current-rectangles nil)
    (setf (output score) str)
    (clear-window win)

    (when (= page 1)
      (if (marks score) (display-marks score score :animation))
      (loop for system in (systems score) do
	(if (marks system) (display-marks system score :animation)))
      (dps-send (format nil "~A~%" (get-output-stream-string str))))

    (map-over-staves #'(lambda (score staff)
			 (let ((page-ctr 1)
			       (cur-objs (staff-data staff)))
			   (loop while (and cur-objs (> page page-ctr)) do
			     (if (page-p (first cur-objs)) (incf page-ctr))
			     (pop cur-objs))
			   (if (and (staff-size staff) (/= (staff-size staff) 1.0))
			       (dps-send (format nil "gsave [~1,3F 0 0 ~1,3F 0 0] concat~%" (staff-size staff) (staff-size staff))))
			   (loop while (and cur-objs
					    (not (page-p (first cur-objs)))) do
			     (let ((obj (pop cur-objs)))
			       (display obj nil score :animation :redraw)
			       (dps-send (format nil "~A~%" (get-output-stream-string str)))))
			   (if (and (staff-size staff) (/= (staff-size staff) 1.0))
			       (dps-send (format nil "grestore~%")))))
		     score)
    (setf current-rectangles old-rects)))

(defun activate-menu (win)
  (loop for menu in (window-menu-rects win) do (draw-menu-item (rect-data menu) menu nil)))


;;; ------------ pull all the foregoing together

(defun dc (name sx0 sy0 x1 y1 &key (title "Untitled") (event-mask nil) (scale 1.0))
  (declare (ignore sx0 sy0))
  (let ((happy t)
	(gsave-sent nil)
	(success 0))
    (unwind-protect
	(let* ((x0 0) (y0 0)
	       (width (max 400 (+ 10 (* scale (- x1 x0)))))
	       (height (max 200 (+ 30 40 (* scale (- y1 y0)))))
	       (base-x (max 0 (min 100 (- screen-width width))))
	       (base-y (max 0 (min 100 (- screen-height height))))
	       (win (create-window base-x base-y (+ base-x width) (+ base-y height) 
				   :vertical-scroll t
				   :horizontal-scroll t
				   :resize t
				   :title title)))
	  
	  (setf current-active-window win)
	  (setf (window-unsaved-changes win) nil)
	  
;	  (draw-vertical-scroll-bar-slot win)
;	  (draw-resize-bar win)
	  
	  ;; next are dependent on relation between score size and window size, and
	  ;; need to be checked after each resize
	  
	  ;; (draw-horizontal-scroll-bar-slot win)
	  ;; (draw-vertical-scroll-buttons win)
	  ;; (draw-horizontal-scroll-buttons win)
	  ;; (draw-vertical-scroll-bar win 1.0 0.0)
	  ;; (draw-horizontal-scroll-bar win 1.0 0.0)

	  (create-menu win)
	  (activate-menu win)
	  (protect-window win)

	  (dps-send (format nil "/Sonata findfont ~D scalefont setfont~%" (size *cmn-score*)))
	  (dps-send (format nil "/insctr 0 def~%"))

	  (dps-send (format nil "gsave [~D 0 0 ~D ~D ~D] concat~%" scale scale 0 0))
	  (setf gsave-sent t)
	  (dps-display-file name)

	  (loop while happy do
	    (dps-flush)

	    (setf success (dps-get-event .5))
	    ;; no recent mouse events -- look for typing in lisp window and deal with it.
	    ;; major danger here is an attempt to call cmn while we're editing the cmn structure:
	    ;; not only will we get very confused, but we can easily kill the window server.
	    (if (zerop success)
		(if (listen)
		    (progn
		      (print (send-message (eval (read-from-string (read-line)))))
		      (terpri)
		      (force-output)))
	      (let ((type (dps-event-type))
		    (mx (dps-event-location-x))
		    (my (dps-event-location-y))
		    (clicks (dps-event-mouse-click)))

		(if (and (= type mouse-up)
			 (plusp clicks))
		    (if (in-close-button win mx my)
			(progn
			  ;; click in the close-button closes window
			  (setf happy nil)
			  (dps-send (format nil "grestore~%"))
			  (dps-send (format nil "~D termwindow~%nulldevice~%" (dps-event-window)))
			  (setf current-active-window nil)
			  (setf current-active-rectangle nil)
			  (setf current-rectangles nil)
			  (dps-flush))
		      (if (in-rect mx my (window-menu-rect win))
			  (perform-menu-action win mx my)))
		  
		  (if (= type mouse-down)
		      (progn
			(if (not (window-active win)) (activate-window win))
			;; make sure we're on top too
			(dps-send (format nil "1 0 ~D orderwindow~%" (dps-event-window)))
;;; if not in-rect mx my user-rect
			(if (in-title-bar win mx my)
			    ;; drag in title-bar moves entire window
			    ;; I don't know what "workspaceWindow" is, or why this code works (there must be some
			    ;;   translation from relative mouse coordinates to movewindow coordinates).  No mention of
			    ;;   this window in /usr/lib/NextStep/windowPackage.ps.
			    (dps-send (format nil "{~D stilldown not {exit} if~%~
				                   workspaceWindow currentmouse ~D sub exch ~D sub exch~%~
				                   ~D movewindow} loop~%"
					      (dps-event-mouse-eventnum)
					      my mx
					      (dps-event-window)))
			  
;;;similarly for resize-bar left/right resize-button
;;;              scroll up/down/left/right if active
			  
;;; if scrolling/scaling -- need to fixup mx/my both directions
			  
			  (let* ((rct (and dpy-pages (find-clicked-rectangle mx my clicks max-depth (dpy-page-data (first dpy-pages)))))
				 (obj (and rct (rect-obj rct)))
				 (animated-matrixless-image (and obj (object-image obj (rect-owner rct))))
				 (animated-image (if animated-matrixless-image
						     (if (rect-matrix rct)
							 (format nil "gsave [~{~1,3F ~}] concat~%~A~%grestore~%"
								 (rect-matrix rct) animated-matrixless-image)
						       animated-matrixless-image))))
			    (when obj
			      ;; show current object in light gray
			      (dps-send (format nil ".667 setgray ~A 0 setgray~%" animated-image))
			      ;; next code pushes the object's displayed image around following the mouse
			      (dps-send (format nil "true setinstance~%~
                                                     {~D stilldown not {exit} if~%~
                                                     gsave ~D currentmouse ~D sub exch ~D sub exch translate~%~
                                                     insctr 10 gt {newinstance} if~%~
                                                     /insctr insctr 1 add def~%~
                                                     insctr 11 gt {/insctr 0 def} if~%~
                                                     ~A grestore}~%~
				                     loop newinstance false setinstance~%"
						;; the insctr foolishness makes the motion smoother looking
						(dps-event-mouse-eventnum)
						(dps-event-window)
						my mx
						animated-image))
			      ;; the assumption is that we got a mouse up to end the dragging, so we need its location
			      (setf type (dps-get-event-no-matter-what))
			      (let* ((new-mx (dps-event-location-x))
				     (new-my (dps-event-location-y))
				     (click (dps-event-mouse-click))
				     (x-incr (- new-mx mx))
				     (y-incr (- new-my my))
				     (redraw nil))
				;; display object in its new location, fix up the dx/dy fields, erase the old image, or redraw score
				
				(if (plusp click)
				    (progn
				      (if current-active-rectangle
					  (dps-send (rect-code current-active-rectangle)))
				      (setf current-active-rectangle rct)
				      (setf (rect-code current-active-rectangle) animated-image))
				  (progn
				    
				    ;; first set the close-button to show that we have unsaved changes
				    (unprotect-window win)
				    (half-close-button-image (- (window-x1 win) 20) (- (window-y1 win) 20))
				    (protect-window win)
				    (setf (window-unsaved-changes win) t)
				    
				    ;; edit the cmn structure
				    (let ((base-dx (/ x-incr (size *cmn-score*)))
					  (base-dy (/ y-incr (size *cmn-score*))))
				      (if (not (rect-matrix rct))
					  (setf redraw (incorporate-changes obj (rect-owner rct) base-dx base-dy))
					(let* ((mat (rect-matrix rct))
					       (trans-x (- base-dx (/ (fifth mat) (size *cmn-score*))))
					       (trans-y (- base-dy (/ (sixth mat) (size *cmn-score*)))))
					  (if (and (zerop (second mat))
						   (zerop (third mat)))
					      ;; the easy case -- just scale and offset, no rotation
					      (setf redraw
						(incorporate-changes obj 
								     (rect-owner rct)
								     (* (/ 1.0 (first mat)) trans-x)
								     (* (/ 1.0 (fourth mat)) trans-y)))
					    ;; in rotated case, we have to get the inverse matrix and apply it to the coordinates
					    (let* ((inverse-matrix (invert-2x2-matrix mat))
						   (new-x (* trans-x (+ (first inverse-matrix) (second inverse-matrix))))
						   (new-y (* trans-y (+ (third inverse-matrix) (fourth inverse-matrix)))))
					      (setf redraw (incorporate-changes obj (rect-owner rct) new-x new-y)))))))
				    
				    ;; fix up the tracking rectangle
				    (edit-rectangle rct x-incr y-incr)
				    ;; erase the previous image and draw the new one
				    (dps-send (format nil "1 setgray ~A 0 setgray~%" animated-image))
				    (if redraw
					(redrawify *cmn-score* current-active-window 1)
				      (dps-send (format nil "gsave ~D ~D translate ~A grestore~%" 
							(- new-mx mx) (- new-my my) animated-image))))))
			      )))))))))
	  )
      (when happy
	(setf current-active-window nil)
	(setf current-active-rectangle nil)
	(setf current-rectangles nil)
	(if gsave-sent (dps-send (format nil "grestore~%")))
	(dps-send (format nil "~D termwindow~%nulldevice~%" (dps-event-window)))
	(dps-flush))
      )))

(defun cmndpy (&optional (file "aaa.eps"))
  (let* ((box (cmn::get-eps-bounding-box file))
	 (width (- (third box) (first box)))
	 (height (- (fourth box) (second box)))
	 (x0 (if (< width 800) 300 0))
	 (x1 (+ x0 width))
	 (y0 (if (< height 600) 200 0))
	 (y1 (+ y0 height)))
    (dc file x0 y0 x1 y1)
    ))
#|
    (display-file 
     file x0 y0 x1 y1 
		  #'(lambda () (dps-send (format nil " ~A ~A translate~%" (- (first box)) (- (second box)))))
		  nil
		  #'(lambda (win) (print "all done")))))
|#
;;; troubles
;;;
;;; need: explicit zoom, scroll, resize, protection from messed up slur structures
;;;       beam/tie/slur/etc fixup, save (eps/cmn), add/delete/edit object somehow
;;;       fixup pitch/leger-line info if note moved a lot, special handlers for slurs/ties/beams/cresc
;;;       smarter box calcs (especially for beams and slurs), and glyph y calcs.
;;;       some way to deal with set-up funcs and vars (as in carl.cmn)
;;;       tags?
;;; menu disappears (partially or completely) after redraw
;;; need refresh after save/redraw etc
